package com.xhkjedu.suser.service.system;

import com.alibaba.fastjson.JSON;
import com.xhkjedu.suser.config.ConfigKey;
import com.xhkjedu.suser.mapper.system.UserLogMapper;
import com.xhkjedu.suser.model.system.TUser;
import com.xhkjedu.suser.model.system.TUserLog;
import com.xhkjedu.suser.utils.IpRegionUtil;
import com.xhkjedu.suser.utils.UserUtil;
import com.xhkjedu.suser.vo.system.UserVo;
import com.xhkjedu.utils.N_Utils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.HashMap;
import java.util.Map;

/**
 * @Description 登录日志
 * @Author WN
 * @Date 2023/2/20 15:54
 **/
@Service
@Slf4j
public class UserLogService {
    @Resource
    private UserLogMapper userLogMapper;

    //保存登录日志
    @Async("asyncPoolTaskExecutor")
    public void saveSyncLoginLog(TUserLog userLog, TUser user, HttpServletRequest request) {
        try {
            userLog.setUserid(user.getUserid());
            userLog.setUsername(user.getUsername());
            userLog.setSchoolid(user.getSchoolid());
            //获取请求ip并解析区域
            String ip = UserUtil.getIpAddress(request);
            String region = null;
            if (N_Utils.isNotEmpty(ip)) {
                IpRegionUtil ipRegionUtil = new IpRegionUtil();
                region = ipRegionUtil.getIpForAddress(ip);
            }
            userLog.setLogip(ip);
            userLog.setLogaddress(region);
            Integer usertype = user.getUsertype();
            String ultype = user.getUltype();
            if (N_Utils.isNotEmpty(ultype) && ultype.equals("p_web") && usertype != null) {
                //网页端学生和教师合并登录，前端传p_web,拼接完整的ultype
                if (usertype == 2) {
                    userLog.setUltype(user.getUltype() + "_s");
                } else {
                    userLog.setUltype(user.getUltype() + "_t");
                }
            } else {
                userLog.setUltype(user.getUltype());
            }

            userLog.setVersionnum(user.getVersionnum());
            int logintime = N_Utils.getSecondTimestamp();
            userLog.setLogtime(logintime);
            userLogMapper.insertSelective(userLog);
        } catch (Exception e) {
            log.error("添加登录日志失败:" + e.getMessage());
        }
    }

    //用户登录信息同步到云平台
    @Async("asyncPoolTaskExecutor")
    public void saveSyncUserLogin(UserVo userVo, TUser user, TUserLog userLog) {
        Map<String, Object> params = new HashMap<>();
        params.put("cloudcode", userVo.getCloudcode());
        params.put("schoolname", userVo.getSchoolname());
        params.put("userid", userVo.getUserid());
        params.put("loginname", userVo.getLoginname());
        params.put("usertype", userVo.getUsertype());
        params.put("device", userLog.getUltype());
        params.put("mac", user.getMac());
        params.put("loginip", userLog.getLogip());
        params.put("logintime", userLog.getLogtime());
        params.put("versioncode", user.getVersionnum());
        params.put("browser", user.getBrowser());
        params.put("env", user.getOs());

        try {
            N_Utils.sendJsonPost(ConfigKey.cloudapi + "rlinfo/save", JSON.toJSONString(params));
        } catch (Exception e) {
            log.error("同步登录日志到云平台失败：" + e.getMessage());
        }
    }
}
